/*
 * Copyright (c) 2004-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: MyLeadAgentStub.java,v 1.24 2007/03/10 03:21:58 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.mylead;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.apache.xmlbeans.XmlException;
import org.ietf.jgss.GSSCredential;
import org.xmlpull.infoset.XmlElement;

import xsul.invoker.gsi.GsiInvoker;
import xsul.invoker.puretls.PuretlsInvoker;
import xsul.wsif.WSIFService;
import xsul.wsif.WSIFServiceFactory;
import xsul.wsif_xsul_soap_gsi.Provider;
import xsul.xwsif_runtime.WSIFClient;
import xsul.xwsif_runtime.XmlBeansWSIFRuntime;
import xsul5.MLogger;
import edu.indiana.dde.mylead.agent.MyLEADAgent_PortType;
import edu.indiana.dde.mylead.agent.xmlbeans.AddNewUserInputParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.AddNewUserInputParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.CreateCollectionInputParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.CreateCollectionInputParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.CreateObjectOutputParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.CreateObjectOutputParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.CreateProjectInputParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.CreateProjectInputParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.MyLeadUserParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.QueryByIdInputParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.QueryByIdInputParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.QueryLeadInputParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.QueryLeadInputParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.QueryLeadWithContextQueryInputParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.QueryLeadWithContextQueryInputParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.QueryOutputParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.QueryOutputParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.SimpleBooleanParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.SimpleBooleanParamsType;
import edu.indiana.dde.mylead.agent.xmlbeans.SimpleStringParamsDocument;
import edu.indiana.dde.mylead.agent.xmlbeans.SimpleStringParamsType;
import edu.indiana.dde.mylead.common.MyLeadBasicContentFilter;
import edu.indiana.dde.mylead.common.MyLeadElementFilter;
import edu.indiana.dde.mylead.common.MyLeadProject;
import edu.indiana.extreme.lead.metadata.LEADresourceDocument;
import edu.indiana.extreme.xbaya.security.SecurityUtil;
import edu.indiana.extreme.xbaya.security.XBayaSecurity;
import edu.indiana.extreme.xbaya.util.WSDLUtil;
import edu.indiana.extreme.xbaya.util.XMLUtil;

/**
 * @author Satoshi Shirasuna
 */
public class MyLeadAgentStub {

    /**
     * HFILTER_TARGET
     */
    public static final String HFILTER_TARGET = "TARGET";

    /**
     * HFILTER_CHILDREN
     */
    public static final String HFILTER_CHILDREN = "CHILDREN";

    private static final MLogger logger = MLogger.getLogger();

    private URI url;

    private GSSCredential proxy;

    private MyLEADAgent_PortType stub;

    /**
     * Creates a MyLeadAgentStub
     * 
     * @param url
     * 
     * @throws MyLeadException
     */
    public MyLeadAgentStub(URI url) throws MyLeadException {
        this(url, null);
    }

    /**
     * Creates a MyLeadAgentStub
     * 
     * @param url
     * @param proxy
     * 
     * @throws MyLeadException
     */
    public MyLeadAgentStub(URI url, GSSCredential proxy) throws MyLeadException {
        this.url = url;
        this.proxy = proxy;
        try {
            WSIFServiceFactory factory = WSIFServiceFactory.newInstance();
            WSIFService service = factory.getService(WSDLUtil.appendWSDLQuary(
                    url).toString(), null, null, null, null);

            if (SecurityUtil.isSecureService(url)) {
                PuretlsInvoker secureInvoker = new GsiInvoker(proxy,
                        XBayaSecurity.getTrustedCertificates());
                Provider secureProvider = new Provider(secureInvoker);
                service.addLocalProvider(secureProvider);
            }
            WSIFClient client = XmlBeansWSIFRuntime.getDefault().newClientFor(
                    service, null);

            // to make it compatible even though some unused operation is
            // removed from the WSDL.
            client.allowMethodsUnmappedToWsdlOperations(true);
            this.stub = (MyLEADAgent_PortType) client
                    .generateDynamicStub(MyLEADAgent_PortType.class);
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * @return The URL.
     */
    public URI getURL() {
        return this.url;
    }

    /**
     * Returns the proxy.
     * 
     * @return The proxy
     */
    public GSSCredential getProxy() {
        return this.proxy;
    }

    /**
     * @param uid
     * @return true if the specified user has an account; false otherwise.
     * @throws MyLeadException
     */
    public boolean doesUserHaveMyLeadAccount(String uid) throws MyLeadException {
        logger.entering(new Object[] { uid });
        try {
            SimpleStringParamsDocument inputDocument = SimpleStringParamsDocument.Factory
                    .newInstance();
            SimpleStringParamsType inputParams = inputDocument
                    .addNewSimpleStringParams();
            inputParams.setSimpleString(uid);

            SimpleBooleanParamsDocument outputDocument = this.stub
                    .doesUserHaveMyLeadAccount(inputDocument);

            SimpleBooleanParamsType outputParams = outputDocument
                    .getSimpleBooleanParams();
            boolean result = outputParams.getSimpleBoolean();
            logger.finest("result: " + result);
            String faultMessage = outputParams.getFaultMessage();
            logger.finest("faultMessage: " + faultMessage);
            if (faultMessage != null) {
                throw new MyLeadException(faultMessage);
            }
            return result;
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * @param adminDN
     * @param userDN
     * @return The query result
     * @throws MyLeadException
     */
    public String addNewUser(String adminDN, String userDN)
            throws MyLeadException {

        try {
            AddNewUserInputParamsDocument inputDocument = AddNewUserInputParamsDocument.Factory
                    .newInstance();
            AddNewUserInputParamsType inputParams = inputDocument
                    .addNewAddNewUserInputParams();
            inputParams.setAdminDN(adminDN);
            MyLeadUserParamsType userParamsType = MyLeadUserParamsType.Factory
                    .newInstance();
            userParamsType.setDN(userDN);
            // XXX there are more parameters to set.
            inputParams.setNewUserInfo(userParamsType);

            QueryOutputParamsDocument outputDocument = this.stub
                    .addNewUser(inputDocument);

            QueryOutputParamsType outputParams = outputDocument
                    .getQueryOutputParams();
            String queryResult = outputParams.getQueryResult();
            logger.finest("queryResult: " + queryResult);
            String faultMessage = outputParams.getFaultMessage();
            logger.finest("faultMessage: " + faultMessage);
            if (faultMessage != null) {
                throw new MyLeadException(faultMessage);
            }
            return queryResult;
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * @param uid
     * @param leadResource
     * @param assignNewResourceID
     * @return The resourceID
     * @throws MyLeadException
     */
    public String createProject(String uid, String leadResource,
            boolean assignNewResourceID) throws MyLeadException {
        try {
            CreateProjectInputParamsDocument inputDocument = CreateProjectInputParamsDocument.Factory
                    .newInstance();
            CreateProjectInputParamsType inputParams = inputDocument
                    .addNewCreateProjectInputParams();
            inputParams.setUID(uid);
            inputParams.setLeadResource(leadResource);
            inputParams.setAssignNewResourceID(assignNewResourceID);

            CreateObjectOutputParamsDocument outputDocument = this.stub
                    .createProject(inputDocument);

            CreateObjectOutputParamsType outputParams = outputDocument
                    .getCreateObjectOutputParams();
            String resourceID = outputParams.getResourceID();
            logger.finest("resourceID: " + resourceID);
            String faultMessage = outputParams.getFaultMessage();
            logger.finest("faultMessage: " + faultMessage);
            if (resourceID != null) {
                return resourceID;
            } else {
                throw new MyLeadException(faultMessage);
            }
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * @param uid
     * @param leadResource
     * @param parentResourceID
     * @param assignNewResourceID
     * @return The resourceID
     * @throws MyLeadException
     */
    @SuppressWarnings("boxing")
    public String createCollection(String uid, String leadResource,
            String parentResourceID, boolean assignNewResourceID)
            throws MyLeadException {
        logger.entering(new Object[] { uid, leadResource, parentResourceID,
                assignNewResourceID });
        try {
            CreateCollectionInputParamsDocument inputDocument = CreateCollectionInputParamsDocument.Factory
                    .newInstance();
            CreateCollectionInputParamsType inputParams = inputDocument
                    .addNewCreateCollectionInputParams();
            inputParams.setUID(uid);
            inputParams.setLeadResource(leadResource);
            inputParams.setParentResourceID(parentResourceID);
            inputParams.setAssignNewResourceID(assignNewResourceID);

            CreateObjectOutputParamsDocument outputDocument = this.stub
                    .createCollection(inputDocument);

            CreateObjectOutputParamsType outputParams = outputDocument
                    .getCreateObjectOutputParams();
            String resourceID = outputParams.getResourceID();
            logger.finest("resourceID: " + resourceID);
            String faultMessage = outputParams.getFaultMessage();
            logger.finest("faultMessage: " + faultMessage);
            if (resourceID != null) {
                return resourceID;
            } else {
                throw new MyLeadException(faultMessage);
            }
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * @param userDN
     * @param limit
     * @param hFilter
     * @param cFilter
     * @param target
     * @return The query result
     * @throws MyLeadException
     */
    public XmlElement queryLead(String userDN, int limit, String hFilter,
            String cFilter, String target) throws MyLeadException {
        logger.entering(new Object[] { userDN, new Integer(limit), hFilter,
                cFilter, target });
        try {
            QueryLeadInputParamsDocument inputDocument = QueryLeadInputParamsDocument.Factory
                    .newInstance();
            QueryLeadInputParamsType inputParams = inputDocument
                    .addNewQueryLeadInputParams();

            inputParams.setUserDN(userDN);
            inputParams.setHFilter(hFilter);
            inputParams.setCFilter(cFilter);
            inputParams.setLimit(limit);
            inputParams.setTarget(target);

            QueryOutputParamsDocument outputDocument = this.stub
                    .queryLead(inputDocument);

            QueryOutputParamsType outputParams = outputDocument
                    .getQueryOutputParams();
            String queryResult = outputParams.getQueryResult();
            // logger.finest("queryResult: " + queryResult);
            String faultMessage = outputParams.getFaultMessage();
            // logger.finest("faultMessage: " + faultMessage);
            if (faultMessage != null) {
                throw new MyLeadException(faultMessage);
            }
            XmlElement resultElement = XMLUtil.stringToXmlElement(queryResult);
            logger.finest("resultElement: "
                    + XMLUtil.xmlElementToString(resultElement));
            return resultElement;
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * @param userDN
     * @param limit
     * @param target
     * @param hFilter
     * @param cFilter
     * @param query
     * @return The query result
     * @throws MyLeadException
     */
    public XmlElement queryLeadWithContextQuery(String userDN, int limit,
            String hFilter, String cFilter, String target, String query)
            throws MyLeadException {
        try {
            QueryLeadWithContextQueryInputParamsDocument inputDocument = QueryLeadWithContextQueryInputParamsDocument.Factory
                    .newInstance();
            QueryLeadWithContextQueryInputParamsType inputParams = inputDocument
                    .addNewQueryLeadWithContextQueryInputParams();

            inputParams.setUserDN(userDN);
            inputParams.setHFilter(hFilter);
            inputParams.setCFilter(cFilter);
            inputParams.setLimit(limit);
            inputParams.setTarget(target);
            inputParams.setQuery(query);

            QueryOutputParamsDocument outputDocument = this.stub
                    .queryLeadwithContextQuery(inputDocument);

            QueryOutputParamsType outputParams = outputDocument
                    .getQueryOutputParams();
            String queryResult = outputParams.getQueryResult();
            // logger.finest("queryResult: " + queryResult);
            String faultMessage = outputParams.getFaultMessage();
            // logger.finest("faultMessage: " + faultMessage);
            if (faultMessage != null) {
                throw new MyLeadException(faultMessage);
            }
            XmlElement resultElement = XMLUtil.stringToXmlElement(queryResult);
            logger.finest("resultElement: "
                    + XMLUtil.xmlElementToString(resultElement));
            return resultElement;
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * @param userDN
     * @param hFilter
     * @param cFilter
     * @param resourceID
     * @return The query result
     * @throws MyLeadException
     */
    public XmlElement queryById(String userDN, String hFilter, String cFilter,
            String resourceID) throws MyLeadException {
        logger.entering(new Object[] { userDN, hFilter, cFilter, resourceID });
        try {
            QueryByIdInputParamsDocument inputDocument = QueryByIdInputParamsDocument.Factory
                    .newInstance();
            QueryByIdInputParamsType inputParams = inputDocument
                    .addNewQueryByIdInputParams();

            inputParams.setUserDN(userDN);
            inputParams.setHFilter(hFilter);
            inputParams.setCFilter(cFilter);
            inputParams.setResourceIDs(resourceID);

            QueryOutputParamsDocument outputDocument = this.stub
                    .queryById(inputDocument);
            QueryOutputParamsType outputParams = outputDocument
                    .getQueryOutputParams();
            String queryResult = outputParams.getQueryResult();
            // logger.finest("queryResult: " + queryResult);
            String faultMessage = outputParams.getFaultMessage();
            // logger.finest("faultMessage: " + faultMessage);
            if (faultMessage != null) {
                throw new MyLeadException(faultMessage);
            }
            XmlElement resultElement = XMLUtil.stringToXmlElement(queryResult);
            logger.finest("resultElement: "
                    + XMLUtil.xmlElementToString(resultElement));
            return resultElement;
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * Lists all projects under a specified user.
     * 
     * @param uid
     * @return The projects
     * @throws MyLeadException
     */
    public ArrayList<MyLeadQueryResultItem> listProjects(String uid)
            throws MyLeadException {
        try {
            int limit = 0;
            String hFilter = HFILTER_TARGET;

            MyLeadElementFilter elementFilter = new MyLeadElementFilter();
            elementFilter.addElement("LEAD", "citation", "LEAD", "title",
                    "title");
            String cFilter = elementFilter.wrapFilter();
            MyLeadProject myLeadProject = new MyLeadProject();
            String target = myLeadProject.wrapParameters();

            XmlElement resultElement = queryLead(uid, limit, hFilter, cFilter,
                    target);

            // resultElement should be like this.
            // 
            // <ml:myQuery timestamp="20070307192347" xmlns:ml="MYLEAD">
            // <ml:project
            // resourceID="urn:uuid:5daa5915-3969-45fb-9dce-f82d5e104b15">
            // <myLeadElement name="title">Project1</myLeadElement>
            // </ml:project>
            // <ml:project
            // resourceID="urn:uuid:afe4ce06-06b1-102a-869a-00e0812d1a4e">
            // <myLeadElement name="title">Default Project</myLeadElement>
            // </ml:project>
            // </ml:myQuery>

            ArrayList<MyLeadQueryResultItem> projects = new ArrayList<MyLeadQueryResultItem>();
            Iterable<XmlElement> projectsElement = resultElement.elements(null,
                    "project");
            for (XmlElement projectElement : projectsElement) {
                MyLeadQueryResultItem project = new MyLeadQueryResultItem();
                String resouceID = projectElement.attributeValue("resourceID");
                logger.finest("resourceID: " + resouceID);
                project.setResouceID(resouceID);

                // Look for myLeadElement that has name="title" as an attribute.
                // e.g.
                // <myLeadElement name="title">Project1</myLeadElement>
                Iterable<XmlElement> myLeadElements = projectElement.elements(
                        null, "myLeadElement");
                for (XmlElement myLeadElement : myLeadElements) {
                    String name = myLeadElement.attributeValue("name");
                    logger.finest("name: " + name);
                    if ("title".equals(name)) {
                        String title = myLeadElement.requiredText();
                        logger.finest("title: " + title);
                        project.setTitle(title);
                        break;
                    }
                }
                projects.add(project);
            }
            return projects;
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * @param uid
     * @param name
     * @return The project
     * @throws MyLeadException
     */
    public String projectNameToID(String uid, String name)
            throws MyLeadException {
        // XXX There must be a direct way to get the project with a specified
        // title from myLEAD.
        ArrayList<MyLeadQueryResultItem> projects = listProjects(uid);
        for (MyLeadQueryResultItem project : projects) {
            String title = project.getTitle();
            if (name.equals(title)) {
                String projectID = project.getResouceID();
                return projectID;
            }
        }
        return null;
    }

    /**
     * List all collections under a specified resource, which can be a project
     * or a collection.
     * 
     * @param uid
     * @param resourceID
     * @return The collections
     * @throws MyLeadException
     */
    public List<MyLeadQueryResultItem> listCollections(String uid,
            String resourceID) throws MyLeadException {
        try {
            String hFilter = HFILTER_CHILDREN;
            MyLeadElementFilter elementFilter = new MyLeadElementFilter();
            elementFilter.addElement("LEAD", "citation", "LEAD", "title",
                    "title");
            String cFilter = elementFilter.wrapFilter();

            XmlElement myQueryElement = queryById(uid, hFilter, cFilter,
                    resourceID);

            ArrayList<MyLeadQueryResultItem> collections = new ArrayList<MyLeadQueryResultItem>();
            Iterable<XmlElement> collectionsElement = myQueryElement.elements(
                    null, "collection");
            for (XmlElement projectElement : collectionsElement) {
                MyLeadQueryResultItem collection = new MyLeadQueryResultItem();
                String resouceID = projectElement.attributeValue("resourceID");
                logger.finest("resourceID: " + resouceID);
                collection.setResouceID(resouceID);
                Iterable<XmlElement> myLeadElements = projectElement.elements(
                        null, "myLeadElement");
                for (XmlElement myLeadElement : myLeadElements) {
                    String name = myLeadElement.attributeValue("name");
                    logger.finest("name: " + name);
                    if ("title".equals(name)) {
                        String title = myLeadElement.requiredText();
                        logger.finest("title: " + title);
                        collection.setTitle(title);
                        break;
                    }
                }
                collections.add(collection);
            }
            return collections;
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }

    /**
     * @param uid
     * @param parentID
     *            This can be either project ID or collection ID.
     * @param collectionName
     * @return The collection ID
     * @throws MyLeadException
     */
    public String collectionNameToID(String uid, String parentID,
            String collectionName) throws MyLeadException {
        // XXX There must be a direct way to get the project with a specified
        // title from myLEAD.
        List<MyLeadQueryResultItem> collections = listCollections(uid, parentID);
        for (MyLeadQueryResultItem collection : collections) {
            String title = collection.getTitle();
            if (collectionName.equals(title)) {
                String resouceID = collection.getResouceID();
                logger.finest("resourceID: " + resouceID);
                return resouceID;
            }
        }
        return null;
    }

    /**
     * @param uid
     * @param collectionID
     * @return The projects
     * @throws MyLeadException
     */
    public LEADresourceDocument getCollection(String uid, String collectionID)
            throws MyLeadException {
        try {
            MyLeadBasicContentFilter cFilter = new MyLeadBasicContentFilter();
            cFilter
                    .setFilter(MyLeadBasicContentFilter.FULL_SCHEMA_CONTENT_FILTER);

            XmlElement myQuery = queryById(uid, HFILTER_TARGET, cFilter
                    .wrapFilter(), collectionID);

            // <myQuery><collection><LEADresource>...

            XmlElement collection = myQuery.element(null, "collection");
            XmlElement leadResource = collection.element(null, "LEADresource");
            LEADresourceDocument metadata = LEADresourceDocument.Factory
                    .parse(XMLUtil.xmlElementToString(leadResource));
            return metadata;
        } catch (MyLeadException e) {
            // No collection with the specified ID found.
            return null;
        } catch (XmlException e) {
            String message = "Couldn't parse the LEAD document.";
            throw new MyLeadException(message, e);
        } catch (RuntimeException e) {
            throw new MyLeadException(e);
        }
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2004-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
